@@ -13,8 +13,6 @@
 
 #include "config.h"
 
-#if !HAVE_HILDON
-
 typedef struct
 {
 	GtkWidget *dialog;
@@ -594,32 +592,24 @@ static void tb_editor_activate_cb(MidoriExtension *extension, MidoriApp *app)
 {
 	KatzeArray *browsers;
 	MidoriBrowser *browser;
-	guint i;
 
 	browsers = katze_object_get_object(app, "browsers");
-	i = 0;
-	while ((browser = katze_array_get_nth_item(browsers, i++)))
+	KATZE_ARRAY_FOREACH_ITEM (browser, browsers)
 		tb_editor_app_add_browser_cb(app, browser, extension);
 	g_signal_connect(app, "add-browser", G_CALLBACK(tb_editor_app_add_browser_cb), extension);
 	g_object_unref(browsers);
 }
 
-#endif
-
 MidoriExtension *extension_init(void)
 {
 	MidoriExtension* extension = g_object_new(MIDORI_TYPE_EXTENSION,
 		"name", _("Toolbar Editor"),
 		"description", _("Easily edit the toolbar layout"),
-		#if !HAVE_HILDON
 		"version", "0.1",
-		#endif
 		"authors", "Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>",
 		NULL);
 
-	#if !HAVE_HILDON
 	g_signal_connect(extension, "activate", G_CALLBACK(tb_editor_activate_cb), NULL);
-	#endif
 
 	return extension;
 }
