@@ -276,19 +276,21 @@ static void
 midori_location_action_popup_position (GtkWidget* popup,
                                        GtkWidget* widget)
 {
+    GdkWindow* window = gtk_widget_get_window (widget);
     gint wx, wy;
     GtkRequisition menu_req;
     GtkRequisition widget_req;
     GdkScreen* screen;
     gint monitor_num;
     GdkRectangle monitor;
+    GtkAllocation allocation;
 
-    gdk_window_get_origin (widget->window, &wx, &wy);
+    gdk_window_get_origin (window, &wx, &wy);
     gtk_widget_size_request (popup, &menu_req);
     gtk_widget_size_request (widget, &widget_req);
 
     screen = gtk_widget_get_screen (widget);
-    monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+    monitor_num = gdk_screen_get_monitor_at_window (screen, window);
     gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
     if (wy + widget_req.height + menu_req.height <= monitor.y + monitor.height
@@ -297,7 +299,8 @@ midori_location_action_popup_position (GtkWidget* popup,
     else
         wy -= menu_req.height;
     gtk_window_move (GTK_WINDOW (popup),  wx, wy);
-    gtk_window_resize (GTK_WINDOW (popup), widget->allocation.width, 1);
+    gtk_widget_get_allocation (widget, &allocation);
+    gtk_window_resize (GTK_WINDOW (popup), allocation.width, 1);
 }
 
 static gboolean
@@ -614,23 +617,25 @@ midori_location_action_finalize (GObject* object)
 static void
 midori_location_action_toggle_arrow_cb (GtkWidget*            widget,
                                         MidoriLocationAction* location_action)
-{    gboolean show = FALSE;
-
-    sqlite3* db;
-    const gchar* sqlcmd;
-    sqlite3_stmt* statement;
+{
+    gboolean show = FALSE;
     gint result;
 
     if (!GTK_IS_BUTTON (widget))
         return;
 
-    db = g_object_get_data (G_OBJECT (location_action->history), "db");
-    sqlcmd = "SELECT uri FROM history LIMIT 1";
-    sqlite3_prepare_v2 (db, sqlcmd, -1, &statement, NULL);
-    result = sqlite3_step (statement);
-    if (result == SQLITE_ROW)
-        show = TRUE;
-    sqlite3_finalize (statement);
+    if (location_action->history != NULL)
+    {
+        sqlite3* db = g_object_get_data (G_OBJECT (location_action->history), "db");
+        const char* sqlcmd = "SELECT uri FROM history LIMIT 1";
+        sqlite3_stmt* statement;
+        sqlite3_prepare_v2 (db, sqlcmd, -1, &statement, NULL);
+        result = sqlite3_step (statement);
+        if (result == SQLITE_ROW)
+            show = TRUE;
+        sqlite3_finalize (statement);
+    }
+
     sokoke_widget_set_visible (widget, show);
     gtk_widget_set_size_request (widget, show ? -1 : 1, show ? -1 : 1);
 }
@@ -640,9 +645,6 @@ midori_location_action_toggle_arrow (MidoriLocationAction* location_action)
 {
     GSList* proxies;
 
-    if (!location_action->history)
-        return;
-
     proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
     for (; proxies != NULL; proxies = g_slist_next (proxies))
     if (GTK_IS_TOOL_ITEM (proxies->data))
@@ -981,7 +983,8 @@ midori_location_action_key_press_event_cb (GtkEntry*    entry,
         }
 
         parent = gtk_widget_get_parent (widget);
-        if (!katze_object_get_boolean (parent, "popup-shown"))
+        if (location_action->history != NULL
+         && !katze_object_get_boolean (parent, "popup-shown"))
             gtk_combo_box_popup (GTK_COMBO_BOX (parent));
         return TRUE;
     }
@@ -1283,7 +1286,7 @@ midori_location_action_entry_popup_cb (GtkComboBox*          combo_box,
     GtkListStore* store;
     gint result;
     const gchar* sqlcmd;
-    static sqlite3_stmt* stmt;
+    static sqlite3_stmt* stmt = NULL;
     gint matches;
 
     store = GTK_LIST_STORE (gtk_combo_box_get_model (combo_box));
@@ -1292,7 +1295,10 @@ midori_location_action_entry_popup_cb (GtkComboBox*          combo_box,
     if (!stmt)
     {
         sqlite3* db;
+
+        g_return_if_fail (location_action->history != NULL);
         db = g_object_get_data (G_OBJECT (location_action->history), "db");
+        g_return_if_fail (db != NULL);
         sqlcmd = "SELECT uri, title FROM history"
                  " GROUP BY uri ORDER BY count() DESC LIMIT ?";
         sqlite3_prepare_v2 (db, sqlcmd, -1, &stmt, NULL);
@@ -1409,9 +1415,8 @@ midori_location_action_connect_proxy (GtkAction* action,
             renderer, midori_location_entry_render_text_cb, action, NULL);
 
         gtk_combo_box_set_active (GTK_COMBO_BOX (entry), -1);
-        if (location_action->history)
-            gtk_container_forall (GTK_CONTAINER (entry),
-                (GtkCallback)midori_location_action_toggle_arrow_cb, action);
+        gtk_container_forall (GTK_CONTAINER (entry),
+            (GtkCallback)midori_location_action_toggle_arrow_cb, action);
         g_signal_connect (entry, "changed",
             G_CALLBACK (midori_location_action_entry_changed_cb), action);
         g_signal_connect (entry, "popup",
@@ -1810,6 +1815,8 @@ midori_location_action_set_security_hint (MidoriLocationAction* location_action,
             #if !HAVE_HILDON
             gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (child),
                 GTK_ICON_ENTRY_SECONDARY, GTK_STOCK_INFO);
+            gtk_icon_entry_set_tooltip (GTK_ICON_ENTRY (child),
+                GTK_ICON_ENTRY_SECONDARY, _("Not verified"));
             #endif
         }
         else if (hint == MIDORI_SECURITY_TRUSTED)
@@ -1819,6 +1826,8 @@ midori_location_action_set_security_hint (MidoriLocationAction* location_action,
             #if !HAVE_HILDON
             gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (child),
                 GTK_ICON_ENTRY_SECONDARY, GTK_STOCK_DIALOG_AUTHENTICATION);
+            gtk_icon_entry_set_tooltip (GTK_ICON_ENTRY (child),
+                GTK_ICON_ENTRY_SECONDARY, _("Verified and encrypted connection"));
             #endif
         }
 
