@@ -196,6 +196,34 @@ midori_preferences_notify_proxy_type_cb (MidoriWebSettings* settings,
 }
 #endif
 
+static void
+midori_preferences_delete_cookies_toggled_cb (GtkToggleButton*   button,
+                                              MidoriWebSettings* settings)
+{
+    gboolean toggled = gtk_toggle_button_get_active (button);
+    g_object_set (settings, "accept-cookies",
+        toggled ? MIDORI_ACCEPT_COOKIES_SESSION : MIDORI_ACCEPT_COOKIES_ALL, NULL);
+}
+
+static void
+midori_preferences_delete_cookies_changed_cb (GtkComboBox*       combo,
+                                              MidoriWebSettings* settings)
+{
+    gint active = gtk_combo_box_get_active (combo);
+    gint max_age;
+    switch (active)
+    {
+    case 0: max_age =   0; break;
+    case 1: max_age =   1; break;
+    case 2: max_age =   7; break;
+    case 3: max_age =  30; break;
+    case 4: max_age = 365; break;
+    default:
+        max_age = 30;
+    }
+    g_object_set (settings, "maximum-cookie-age", max_age, NULL);
+}
+
 #if HAVE_OSX
 static void
 midori_preferences_toolbutton_clicked_cb (GtkWidget* toolbutton,
@@ -281,8 +309,8 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     if ((header = sokoke_xfce_header_new (icon_name,
         gtk_window_get_title (GTK_WINDOW (preferences)))))
     {
-        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (preferences)->vbox),
-            header, FALSE, FALSE, 0);
+        GtkWidget* vbox = gtk_dialog_get_content_area (GTK_DIALOG (preferences));
+        gtk_box_pack_start (GTK_BOX (vbox), header, FALSE, FALSE, 0);
         gtk_widget_show_all (header);
     }
     _preferences = KATZE_PREFERENCES (preferences);
@@ -403,13 +431,9 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     #if !HAVE_HILDON
     button = katze_property_proxy (settings, "enable-scripts", NULL);
     INDENTED_ADD (button);
-    button = katze_property_proxy (settings, "open-panels-in-windows", NULL);
-    SPANNED_ADD (button);
     button = katze_property_proxy (settings, "enable-plugins", NULL);
-    INDENTED_ADD (button);
-    #endif
-    button = katze_property_proxy (settings, "find-while-typing", NULL);
     SPANNED_ADD (button);
+    #endif
     button = katze_property_proxy (settings, "zoom-text-and-images", NULL);
     INDENTED_ADD (button);
     #if WEBKIT_CHECK_VERSION (1, 1, 11)
@@ -424,9 +448,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_button_set_label (GTK_BUTTON (button), _("Enable Spell Checking"));
     gtk_widget_set_tooltip_text (button, _("Enable spell checking while typing"));
     INDENTED_ADD (button);
-    button = gtk_label_new (_("Spelling dictionaries:"));
-    INDENTED_ADD (button);
-    entry = katze_property_proxy (settings, "spell-checking-languages", NULL);
+    entry = katze_property_proxy (settings, "spell-checking-languages", "languages");
     /* i18n: The example should be adjusted to contain a good local default */
     gtk_widget_set_tooltip_text (entry, _("A comma separated list of "
        "languages to be used for spell checking, for example \"en_GB,de_DE\""));
@@ -450,10 +472,6 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     INDENTED_ADD (label);
     button = katze_property_proxy (settings, "open-new-pages-in", NULL);
     SPANNED_ADD (button);
-    label = katze_property_label (settings, "open-external-pages-in");
-    INDENTED_ADD (label);
-    button = katze_property_proxy (settings, "open-external-pages-in", NULL);
-    SPANNED_ADD (button);
     #if !HAVE_HILDON
     button = katze_property_proxy (settings, "always-show-tabbar", NULL);
     INDENTED_ADD (button);
@@ -505,30 +523,60 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     SPANNED_ADD (button);
     label = katze_property_label (settings, "preferred-languages");
     INDENTED_ADD (label);
-    entry = katze_property_proxy (settings, "preferred-languages", NULL);
+    entry = katze_property_proxy (settings, "preferred-languages", "languages");
     SPANNED_ADD (entry);
 
-
     /* Page "Privacy" */
     PAGE_NEW (GTK_STOCK_INDEX, _("Privacy"));
     FRAME_NEW (_("Web Cookies"));
-    label = katze_property_label (settings, "accept-cookies");
-    INDENTED_ADD (label);
-    button = katze_property_proxy (settings, "accept-cookies", NULL);
+    button = gtk_check_button_new_with_mnemonic (_("Delete cookies when quitting Midori"));
+    INDENTED_ADD (button);
+    if (katze_object_get_enum (settings, "accept-cookies") == MIDORI_ACCEPT_COOKIES_SESSION)
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+    g_signal_connect (button, "toggled",
+        G_CALLBACK (midori_preferences_delete_cookies_toggled_cb), settings);
+    button = gtk_combo_box_new_text ();
+    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 hour"));
+    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 day"));
+    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 week"));
+    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 month"));
+    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 year"));
+    {
+        gint max_age = katze_object_get_int (settings, "maximum-cookie-age");
+        guint active;
+        switch (max_age)
+        {
+        case   0: active = 0; break;
+        case   1: active = 1; break;
+        case   7: active = 2; break;
+        case  30: active = 3; break;
+        case 365: active = 4; break;
+        default:
+            active = 3;
+        }
+        gtk_combo_box_set_active (GTK_COMBO_BOX (button), active);
+    }
+    g_signal_connect (button, "changed",
+        G_CALLBACK (midori_preferences_delete_cookies_changed_cb), settings);
     SPANNED_ADD (button);
-    label = katze_property_label (settings, "maximum-cookie-age");
-    INDENTED_ADD (label);
-    entry = katze_property_proxy (settings, "maximum-cookie-age", NULL);
-    SPANNED_ADD (entry);
-    label = gtk_label_new (_("days"));
-    SPANNED_ADD (label);
-    #if WEBKIT_CHECK_VERSION (1, 1, 8)
-    INDENTED_ADD (katze_property_proxy (settings, "enable-html5-database", NULL));
-    SPANNED_ADD (katze_property_proxy (settings, "enable-html5-local-storage", NULL));
-    #endif
+    {
+        gchar* markup = g_strdup_printf ("<span size=\"smaller\">%s</span>",
+            _("Cookies store login data, saved games, "
+              "or user profiles for advertisement purposes."));
+        label = gtk_label_new (NULL);
+        gtk_label_set_markup (GTK_LABEL (label), markup);
+        g_free (markup);
+    }
+    FILLED_ADD (label);
     #if WEBKIT_CHECK_VERSION (1, 1, 13)
     INDENTED_ADD (katze_property_proxy (settings, "enable-offline-web-application-cache", NULL));
     #endif
+    #if WEBKIT_CHECK_VERSION (1, 1, 8)
+    SPANNED_ADD (katze_property_proxy (settings, "enable-html5-local-storage", NULL));
+    #if !WEBKIT_CHECK_VERSION (1, 1, 14)
+    INDENTED_ADD (katze_property_proxy (settings, "enable-html5-database", NULL));
+    #endif
+    #endif
     FRAME_NEW (_("History"));
     button = katze_property_label (settings, "maximum-history-age");
     INDENTED_ADD (button);
@@ -536,6 +584,4 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     SPANNED_ADD (button);
     label = gtk_label_new (_("days"));
     SPANNED_ADD (label);
-    button = katze_property_proxy (settings, "remember-last-downloaded-files", NULL);
-    INDENTED_ADD (button);
 }
