@@ -12,7 +12,11 @@
 #ifndef __MIDORI_BOOKMARKS_H__
 #define __MIDORI_BOOKMARKS_H__
 
+#include "config.h"
+
+#include <sqlite3.h>
 #include <gtk/gtk.h>
+#include <katze/katze.h>
 
 G_BEGIN_DECLS
 
@@ -38,6 +42,16 @@ midori_bookmarks_get_type               (void);
 GtkWidget*
 midori_bookmarks_new                    (void);
 
+void
+midori_bookmarks_insert_item_db (sqlite3*     db,
+                                 KatzeItem*   item,
+                                 const gchar* folder);
+
+void
+midori_bookmarks_import_array_db (sqlite3*       db,
+                                  KatzeArray*    array,
+                                  const gchar*   folder);
+
 G_END_DECLS
 
 #endif /* __MIDORI_BOOKMARKS_H__ */
